/*
 * DefaultBootstrapLoader.java
 *
 * Created on 23. August 2004, 21:32
 */

package org.jconfig.bootstrap;

import java.io.IOException;
import java.io.InputStream;
import org.jconfig.*;
import org.jconfig.handler.*;
import org.jconfig.error.*;
import org.jconfig.utils.ResourceLocator;
import java.util.Vector;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
/**
 * This BootstrapLoader will read in an XML file called "bootstrap.xml"
 * and parse the content. Then it will load every configuration  with the
 * defined handler and (optional) parser.<br/>
 * <B>Note:</B> This is currently not implemented and this loader works
 * exactly like the DefaultBootstrapLoader!!!
 *
 * @author  Andreas Mecky andreasmecky@yahoo.de
 * @author  Terry Dye terrydye@yahoo.com
 */
public class XMLBootstrapLoader implements BootstrapLoader {
    
    /**
     *
     */
    public XMLBootstrapLoader() {
    }
    
    public LoadedItem[] load() throws ConfigurationManagerException {
        Vector configs = new Vector();
        String fileName = "config.xml";
        InputStreamHandler ish = new InputStreamHandler(fileName);
        Configuration config = ish.load("default");
        if ( config != null ) {
            LoadedItem li = new LoadedItem(config.getConfigName(),config,ish);
            configs.add(li);
        }
        return (LoadedItem[])configs.toArray(new LoadedItem[0]);
    }
    
    private LoadedItem[] readBootstrap() {
        InputStream is = null;
        try {
            is = new ResourceLocator("bootstrap.xml").getInputStream();
            if ( is == null ) {
                ErrorReporter.getErrorHandler().reportError("File not found:bootstrap.xml");
                return null;
            }
        } catch (IOException e) {
            ErrorReporter.getErrorHandler().reportError("Cannot load file:bootstrap.xml",e);
            return null;
        }
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            ErrorReporter.getErrorHandler().reportError("The parser cannot create a new document builder",pce);
            return null;
        }
        Document doc = null;
        try {
            doc = db.parse(is);
        } catch (SAXException se) {
            ErrorReporter.getErrorHandler().reportError("The parser cannot parse the file",se);
            return null;
        } catch (IOException ioe) {
            ErrorReporter.getErrorHandler().reportError("The parser cannot open the file",ioe);
            return null;
        }
        return null;
    }
    
}
